from typing import Literal

import pandas as pd

from ...api import api

from .common import create_expenditure_tiers, filter_urban_rural


INDEX = [
    "Total",
    "Managers",
    "Professionals",
    "Technicians and Associate Professionals",
    "Clerical Support Workers",
    "Service and Sales Workers",
    "Skilled Agricultural, Forestry and Fishery Workers",
    "Craft and Related Trades Workers",
    "Plant and Machine Operators, and Assemblers",
    "Elementary Occupations",
    "Unknown or Other",
    "Unemployed",
]


def create_table_for_a_year(
    year: int, urban_rural: Literal["urban", "rural"]
) -> pd.DataFrame:
    activity_status = (
        api.load_table("members_properties", years=1401)
        .set_index(["Year", "ID", "Member_Number"])
        .loc[lambda df: df["Relationship"] == "Head", ["Activity_Status"]]
    )

    income_table = pd.concat(
        [
            api.load_table("employment_income", years=year),
            api.load_table("self_employed_income", years=year),
        ],
        ignore_index=True,
    )
    occupations = (
        income_table.drop_duplicates(["ID", "Member_Number"], keep="first")
        .pipe(api.add_classification, target="Occupation_Code", aspects="title")
        .set_index(["Year", "ID", "Member_Number"])
        .loc[:, ["Occupation"]]
    )

    occupations = activity_status.join(occupations, how="left").droplevel(-1)
    filt = occupations["Activity_Status"] != "Employed"
    occupations.loc[filt, "Occupation"] = "Unemployed"
    filt = occupations["Occupation"].isna() & (
        occupations["Activity_Status"] == "Employed"
    )
    occupations.loc[filt, "Occupation"] = "Unknown or Other"

    occupations = occupations["Occupation"]
    occupations = occupations.replace({"Armed Forces Occupations": "Unknown or Other"})

    table = pd.concat(
        [
            occupations,
            create_expenditure_tiers(urban_rural=urban_rural, year=year),
            api.load_table("Weight", years=year).set_index(["Year", "ID"])["Weight"],
        ],
        axis="columns",
    )

    table = filter_urban_rural(table, urban_rural)

    weight_table = (
        table.groupby(["Occupation", "Expenditure_Tier"], observed=True)["Weight"]
        .sum()
        .unstack()
    )
    index_weight_sum = weight_table.sum(axis="index")
    columns_weight_sum = weight_table.sum(axis="columns")
    col_sum = columns_weight_sum.div(columns_weight_sum.sum()).mul(100).rename("Total")
    main_table = weight_table.div(index_weight_sum).mul(100)
    main_table = pd.concat([col_sum, main_table], axis="columns")
    main_table = pd.concat(
        [
            main_table.sum().to_frame("Total").T,
            main_table,
        ]
    )

    main_table = main_table.fillna(0)
    main_table = main_table.loc[INDEX]
    return main_table


def main(years: list[int], urban_rural: Literal["urban", "rural"]):
    if len(years) == 1:
        return create_table_for_a_year(years[0], urban_rural)
    raise ValueError



import pandas as pd

from ..api import api


def create_calorie_requirement_table() -> pd.DataFrame:
    internal_data_path = api.defaults.package_dir / "internal_data"
    calorie_requirements = pd.read_csv(
        internal_data_path / "nnftri_calorie_requirements.csv"
    )
    return (
        calorie_requirements
        .loc[lambda df: df["Age_Interval"].ne("Average")]
        .loc[lambda df: df["Sex"].ne("Average")]
        .pipe(
            lambda df: df.join(
                df["Age_Interval"]
                .str.split("-", expand=True)
                .set_axis(["min_age", "max_age"], axis="columns")
                .astype(int)
            )
        )
        .loc[lambda df: df.index.repeat(df["max_age"].sub(df["min_age"]).add(1))]
        .reset_index()
        .assign(
            cumcount=lambda df:
            df.groupby(["Age_Interval", "Sex"])["Calorie_Requirement"].cumcount(),
            Age=lambda df: df["min_age"].add(df["cumcount"]),
        )
        .set_index(["Age", "Sex"])
        .loc[:, ["Calorie_Requirement"]]
    )


def calorie_based_equivalence_scale(table: pd.DataFrame) -> pd.DataFrame:
    internal_data_path = api.defaults.package_dir / "internal_data"
    calorie_requirements = pd.read_csv(
        internal_data_path / "nnftri_calorie_requirements.csv"
    )
    average_calorie_requirement = (
        calorie_requirements
        .loc[lambda df: df["Age_Interval"].eq("Average")]
        .loc[lambda df: df["Sex"].eq("Average")]
        .loc[:, "Calorie_Requirement"]
        .iloc[0]
    )

    return (
        table
        .join(create_calorie_requirement_table(), on=["Age", "Sex"])
        .groupby(["Year", "ID"])[["Calorie_Requirement"]].sum()
        .div(average_calorie_requirement)
        .rename(columns={"Calorie_Requirement": "Adult_Equivalent_Scale_NNFTRI"})
        .reset_index()
    )



import pandas as pd

from ..api import api


def calculate_weight(table: pd.DataFrame) -> pd.DataFrame:
    years = table["Year"].unique()
    year = years[0]
    internal_data_path = api.defaults.package_dir / "internal_data"
    if year >= 1373:
        province_weights_path = internal_data_path / "province_weights_1373.csv"
    elif year > 1371:
        province_weights_path = internal_data_path / "province_weights_1371.csv"

    province_weights = pd.read_csv(province_weights_path, index_col=0)
    province_weight_share = province_weights.stack().rename("Weight_Share")

    if year > 1370:
        census_year = 1370
    census_household_count_path = internal_data_path / "census_household_count.csv"
    census_household_count = pd.read_csv(census_household_count_path)
    household_count = (
        census_household_count
        .loc[lambda df: df["Year"].eq(census_year), ["Urban", "Rural"]]
        .iloc[0]
        .rename("Household_Count")
    )

    return (
        table
        .pipe(api.add_attribute, name="Province", aspects="code")
        .pipe(api.add_attribute, name="Urban_Rural")
        .assign(
            Sample_Count=lambda df:
            df.groupby(["Province", "Urban_Rural"])["ID"].transform("count")
        )
        .join(province_weight_share, on=["Province", "Urban_Rural"])
        .join(household_count, on=["Urban_Rural"])
        .assign(
            Weight=lambda df:
            df["Household_Count"].div(df["Sample_Count"]).mul(df["Weight_Share"])
        )
        .loc[:, ["Year", "ID", "Weight"]]
    )



from pathlib import Path
import io
from typing import Literal, Optional
from zipfile import ZipFile, ZIP_DEFLATED

import pandas as pd
from pandas.io.formats.style import Styler
import yaml

import hbsir

SUB_VERSION = 3


POSSESSIONS = [
    "Bathroom",
    "Kitchen",
    "Car",
    "Motorcycle",
    "TV",
    "PC",
    "Internet",
    "Phone",
    "Cellphone",
    "Refrigerator",
    "Freezer",
    "Freezer_Refrigerator",
    "Oven",
    "Vacuum_Cleaner",
    "Washing_Machine",
    "Sewing_Machine",
    "Dishwasher",
    "Microwave_Oven",
    "Pipe_Water",
    "Electricity",
    "Natural_Gas",
    "Sewerage",
    "Evaporative_AC",
    "Central_AC",
    "Water_Heater",
    "Central_Heating",
    "Split_AC",
]


def create_household_information_part(year: int) -> pd.DataFrame:
    return (
        hbsir.load_table("household_information", years=year)
        .assign(Season_Number=lambda df: df["Season"].cat.codes.add(1))
        .pipe(
            hbsir.add_attribute,
            name="Urban_Rural",
            aspects=["code", "name"],
            column_names=["Urban_Rural_Code", "Urban_Rural"],
        )
        .assign(Urban_Rural_Code=lambda s: s["Urban_Rural_Code"].astype(str))
        .pipe(
            hbsir.add_attribute,
            name="Province",
            aspects=["code", "name"],
            column_names=["Province_Code", "Province"],
        )
        .assign(Province_Code=lambda s: s["Province_Code"].astype(str).str.pad(2, "left", "0"))
        .pipe(
            hbsir.add_attribute,
            name="County",
            aspects=["code", "name"],
            column_names=["County_Code", "County"],
        )
        .assign(County_Code=lambda s: s["County_Code"].astype(str).str.pad(4, "left", "0"))
        .set_index(["Year", "ID"])
        .loc[
            :,
            [
                "Season_Number",
                "Season",
                "Urban_Rural_Code",
                "Urban_Rural",
                "Province_Code",
                "Province",
                "County_Code",
                "County",
                "Weight",
            ]
        ]
    )

def fillna_with_false(df: pd.DataFrame) -> pd.DataFrame:
    df.loc[:, "Car":"Sewerage"] = df.loc[:, "Car":"Sewerage"].fillna(False)
    return df

def create_house_specifications_table(year: int) -> pd.DataFrame:
    return (
        hbsir.load_table("house_specifications", years=year)
        .pipe(fillna_with_false)
        .set_index(["Year", "ID"])
        .assign(
            Structure_Type = lambda df:
            df["Structure_Type"]
            .astype(str)
            .where(df["Structure_Type"].ne("Other"), df["Structure_Main_Materials"])
            .astype("category")
        )
        .assign(TV=lambda df: df["Black_and_White_TV"] | df["Color_TV"])
        .astype({p: "Int8" for p in POSSESSIONS})
        .rename(columns={p: f"Has_{p}" for p in POSSESSIONS})
        .loc[
            :,
            [
                # "Dwelling_Type",
                "Tenure",
                "Structure_Type",
                "House_Area",
                "Number_of_Rooms",
            ] +
            [f"Has_{p}" for p in POSSESSIONS] +
            [
                "Cooking_Fuel",
                "Heating_Fuel",
                "Hotwater_Fuel",
            ]
        ]
    )

def create_demographics_table(year: int) -> pd.DataFrame:
    return (
        hbsir.load_table("members_properties", years=year)
        .groupby(["Year", "ID"])
        .aggregate(
            Family_Size = pd.NamedAgg("Member_Number", "count"),
            Number_of_Spouses = pd.NamedAgg("Relationship", lambda s: s.eq("Spouse").sum()),
            Number_of_Childs = pd.NamedAgg("Relationship", lambda s: s.eq("Child").sum()),
            Number_of_Active_Members = pd.NamedAgg("Activity_Status", lambda s: s.eq("Employed").sum()),
            Number_of_Literate_Members = pd.NamedAgg("Is_Literate", "sum"),
            Number_of_Students = pd.NamedAgg("Is_Student", "sum"),
        )
    )

def create_equivalence_scale(year: int) -> pd.DataFrame:
    return (
        hbsir.load_table("Equivalence_Scale", years=year)
        .set_index(["Year", "ID"])
        .loc[:, ["OECD", "OECD_Modified", "Square_Root"]]
        .rename(columns=lambda n: f"{n}_Equivalence_Scale")
    )


def create_member_property(year: int, relationship: Literal["Head", "Spouse"] = "Head") -> pd.DataFrame:
    return (
        hbsir.load_table("members_properties", years=year)
        .loc[lambda df: df["Relationship"].eq(relationship)]
        .join(
            hbsir.load_table("Years_of_Schooling", years=year)
                .set_index(["Year", "ID"])["Years_of_Schooling"],
            on=["Year", "ID"],
            how="left",
        )
        .sort_values("Age")
        .drop_duplicates("ID", keep="last")
        .set_index(["Year", "ID"])
        .astype({"Is_Literate": "Int8", "Is_Student": "Int8"})
        .loc[
            :,
            [
                "Sex",
                "Age",
                "Is_Literate",
                "Is_Student",
                "Education_Level",
                "Years_of_Schooling",
                "Marital_Status",
                "Activity_Status"
            ]
        ]
        .rename(columns=lambda n: f"{relationship}_{n}")
    )


def create_expenditure_totals(year: int) -> pd.DataFrame:
    return (
        hbsir.load_table("Expenditures", years=year)
        .groupby(["Year", "ID"])[["Net_Expenditure", "Gross_Expenditure"]].sum()
    )


def create_expenditure_level_1(year: int) -> pd.DataFrame:
    return (
        hbsir.load_table("Expenditures", years=year)
        .pipe(hbsir.add_classification, aspects=["name"], levels=[1], column_names=["COICOP_L1"])
        .dropna(subset="COICOP_L1")
        .groupby(["Year", "ID", "COICOP_L1"])["Gross_Expenditure"].sum()
        .unstack(-1, 0)
    )


def create_food_expenditure_level_3(year: int) -> pd.DataFrame:
    return (
        hbsir.load_table("Expenditures", years=year)
        .loc[lambda df: df["Table_Name"].eq("food")]
        .pipe(hbsir.add_classification, aspects=["name"], levels=[3], column_names=["COICOP_L3"])
        .dropna(subset="COICOP_L3")
        .groupby(["Year", "ID", "COICOP_L3"])["Gross_Expenditure"].sum()
        .unstack(-1, 0)
    )


def create_income_table(year: int) -> pd.DataFrame:
    return (
        hbsir.load_table("Income_Breakdown", years=year)
        .pivot(index=["Year", "ID"], columns="Income_Type", values="Income")
        .fillna(0)
    )

def create_level3_aggs(table: pd.DataFrame) -> pd.DataFrame:
    return (
        table
        .assign(
            Cash_Employment_Income=lambda df: df[[
                "Cash_Cooperative", "Cash_Private", "Cash_Public",
            ]].sum(axis="columns"),
            NonCash_Employment_Income=lambda df: df[[
                "NonCash_Cooperative", "NonCash_Private", "NonCash_Public",
            ]].sum(axis="columns"),

            **{
                "Cash_Self-Employed_Income": lambda df: df[[
                    "Cash_Agricultural", "Cash_NonAgricultural", "Cash_Home_Production",
                ]].sum(axis="columns"),
                "NonCash_Self-Employed_Income": lambda df: df[[
                    "NonCash_Agricultural", "NonCash_NonAgricultural", "NonCash_HomeProduction"
                ]].sum(axis="columns"),
            },

            NonCash_ImputedRent = lambda df: df[[
                "NonCash_ImputedRent_Mortgage", "NonCash_ImputedRent_Ownership"
            ]].sum(axis="columns"),

            Cash_Aid_and_Transfer=lambda df: df[[
                "Cash_Aid", "Cash_Subsidy", "Cash_Transfer",
            ]].sum(axis="columns"),
            NonCash_Aid_and_Transfer=lambda df: df[[
                "NonCash_Donation",
            ]].sum(axis="columns"),

            Cash_Other_Passive_Income=lambda df: df[[
                "Cash_Interest", "Cash_Retirement",
            ]].sum(axis="columns"),
        )
        .loc[
            :,
            [
                "Cash_Employment_Income",
                "NonCash_Employment_Income",
                "Cash_Self-Employed_Income",
                "NonCash_Self-Employed_Income",
                "Cash_Rent",
                "NonCash_ImputedRent",
                "Cash_Aid_and_Transfer",
                "NonCash_Aid_and_Transfer",
                "Cash_Other_Passive_Income",
            ]
        ]
    )

def create_level2_aggs(table: pd.DataFrame) -> pd.DataFrame:
    return (
        table
        .assign(
            Employment_Income=lambda df: df[[
                "Cash_Employment_Income", "NonCash_Employment_Income"
            ]].sum(axis="columns"),

            **{
                "Self-Employed_Income": lambda df: df[[
                    "Cash_Self-Employed_Income", "NonCash_Self-Employed_Income",
                ]].sum(axis="columns"),
            },

            Rent = lambda df: df[[
                "NonCash_ImputedRent", "Cash_Rent"
            ]].sum(axis="columns"),

            Aid_and_Transfer=lambda df: df[[
                "Cash_Aid_and_Transfer", "NonCash_Aid_and_Transfer",
            ]].sum(axis="columns"),

            Other_Passive_Income=lambda df: df[[
                "Cash_Other_Passive_Income",
            ]].sum(axis="columns"),
        )
        .loc[
            :,
            [
                "Employment_Income",
                "Self-Employed_Income",
                "Rent",
                "Aid_and_Transfer",
                "Other_Passive_Income",
            ]
        ]
    )

def create_level1_aggs(table: pd.DataFrame) -> pd.DataFrame:
    return (
        table
        .assign(
            Active_Income = lambda df: df[[
                "Employment_Income", "Self-Employed_Income"
            ]].sum(axis="columns"),
            Passive_Income = lambda df: df[[
                "Aid_and_Transfer", "Rent", "Other_Passive_Income"
            ]].sum(axis="columns"),
            Income = lambda df: df[["Active_Income", "Passive_Income"]].sum(axis="columns"),
        )
        .loc[
            :,
            [
                "Active_Income",
                "Passive_Income",
                "Income",
            ]
        ]
    )


def add_decile(
    df: pd.DataFrame,
    on_variable: str,
    equivalence_scale: str,
    groupby: Optional[list[str]] = None,
) -> pd.DataFrame:
    col_name = f"{on_variable}_Decile_{equivalence_scale}"
    if groupby is not None:
        if "Urban_Rural" in groupby:
            col_name += "_UrbanRural"
        if "Province" in groupby:
            col_name += "_Province"
    return (
        df
        .pipe(
            hbsir.calculate.add_decile,
            equivalence_scale=equivalence_scale,
            on_variable=on_variable,
            quantile_column_name=col_name,
        )
    )


def create_deciles(year: int) -> pd.DataFrame:
    return (
        hbsir.load_table("household_information", years=year)
        .pipe(add_decile, "Income", "OECD")
        .pipe(add_decile, "Income", "OECD_Modified")
        .pipe(add_decile, "Gross_Expenditure", "OECD")
        .pipe(add_decile, "Gross_Expenditure", "OECD_Modified")
        .set_index(["Year", "ID"])
        .loc[:, lambda df: [c for c in df.columns if "Decile" in c]]
    )


def create_employment_job_data(
    year: int, relationship: Literal["Head", "Spouse"] = "Head"
) -> pd.DataFrame:
    table = (
        hbsir.load_table("members_properties", years=year)
        .loc[lambda df: df["Relationship"].eq(relationship)]
        .loc[:, ["Year", "ID", "Member_Number"]]
        .join(
            hbsir.load_table("employment_income", years=year)
                .set_index(["Year", "ID", "Member_Number"]),
            on=["Year", "ID", "Member_Number"],
            how="inner",
        )
        .drop(columns="Member_Number")
        .rename(
            columns={
                "Annual_Net_Income": "Income",
                "Working_Day_per_Week": "Days_per_Week",
                "Working_Hours_per_Day": "Hours_per_Day",
            }
        )
        .assign(
            Occupation_Code=lambda df: df["Occupation_Code"].astype(str).str.pad(4, "left", "0"),
            Industry_Code=lambda df: df["Industry_Code"].astype(str).str.pad(5, "left", "0"),
        )
        .sort_values(["Year", "ID", "Income"])
        .drop_duplicates(["Year", "ID"], keep="last")
        .set_index(["Year", "ID"])
        .loc[:, ["Employment_Status", "Sector", "Occupation_Code", "Industry_Code", "Days_per_Week", "Hours_per_Day", "Income"]]
        .rename(columns=lambda n: f"{relationship}_Employment_{n}")
    )
    assert table.index.duplicated().sum() == 0
    return table


def create_self_employed_job_data(
    year: int, relationship: Literal["Head", "Spouse"] = "Head"
) -> pd.DataFrame:
    table = (
        hbsir.load_table("members_properties", years=year)
        .loc[lambda df: df["Relationship"].eq(relationship)]
        .loc[:, ["Year", "ID", "Member_Number"]]
        .join(
            hbsir.load_table("self_employed_income", years=year)
                .set_index(["Year", "ID", "Member_Number"]),
            on=["Year", "ID", "Member_Number"],
            how="inner",
        )
        .drop(columns="Member_Number")
        .rename(
            columns={
                "Profit": "Income",
                "Working_Day_per_Week": "Days_per_Week",
                "Working_Hours_per_Day": "Hours_per_Day",
            }
        )
        .assign(
            Occupation_Code=lambda df: df["Occupation_Code"].astype(str).str.pad(4, "left", "0"),
            Industry_Code=lambda df: df["Industry_Code"].astype(str).str.pad(5, "left", "0"),
        )
        .sort_values(["Year", "ID", "Income"])
        .drop_duplicates(["Year", "ID"], keep="last")
        .set_index(["Year", "ID"])
        .loc[:, ["Employment_Status", "Occupation_Code", "Industry_Code", "Days_per_Week", "Hours_per_Day", "Income"]]
        .rename(columns=lambda n: f"{relationship}_Self_Employed_{n}")
    )
    assert table.index.duplicated().sum() == 0
    return table


def create_tables(year: int) -> list[pd.DataFrame]:
    income_table = create_income_table(year)
    income_level3_aggs = create_level3_aggs(income_table)
    income_level2_aggs = create_level2_aggs(income_level3_aggs)
    income_level1_aggs = create_level1_aggs(income_level2_aggs)

    tables = [
        create_household_information_part(year),
        create_house_specifications_table(year),
        create_demographics_table(year),
        create_equivalence_scale(year),
        create_member_property(year, "Head"),
        create_member_property(year, "Spouse"),
        create_expenditure_totals(year),
        create_expenditure_level_1(year),
        create_food_expenditure_level_3(year),
        income_table.drop(columns="Cash_Rent"),
        income_level3_aggs,
        income_level2_aggs,
        income_level1_aggs,
        create_deciles(year),
        create_employment_job_data(year),
        create_self_employed_job_data(year),
    ]
    return tables


def create_summary_table(tables: list[pd.DataFrame], columns: Optional[list[str]] = None) -> pd.DataFrame:
    summary_table = (
        pd.concat(tables, axis="columns", join="outer")
        .dropna(subset=["Weight"])
        .sort_index()
        .reset_index()
    )
    if columns is not None:
        summary_table = summary_table.loc[:, columns]
    return summary_table


def create_styled_table(tables: list[pd.DataFrame], columns: Optional[list[str]] = None) -> Styler:
    styled_df = (
        create_summary_table(tables, columns)
        .style

        .set_properties(
            ["Year", "ID"],
            **{
                "background-color": "#D9D9D9",
                "border": "2px solid #A6A6A6",
                "font-weight": "bold",
            }
        )
        .set_properties(
            tables[0].columns,
            **{
                "background-color": "#DAEEF3",
                "border": "1px solid #92CDDC",
            }
        )
        .set_properties(
            tables[1].columns,
            **{
                "background-color": "#DDD9C4",
                "border": "1px solid #948A54",
            }
        )
        .set_properties(
            tables[2].columns,
            **{
                "background-color": "#DCE6F1",
                "border": "1px solid #95B3D7",
            }
        )
        .set_properties(
            tables[3].columns,
            **{
                "background-color": "#B8CCE4",
                "border": "1px solid #95B3D7",
            }
        )
        .set_properties(
            tables[4].columns,
            **{
                "background-color": "#DCE6F1",
                "border": "1px solid #95B3D7",
            }
        )
        .set_properties(
            tables[5].columns,
            **{
                "background-color": "#B8CCE4",
                "border": "1px solid #95B3D7",
            }
        )
        .set_properties(
            tables[6].columns,
            **{
                "background-color": "#DA9694",
                "border": "1px solid #632523",
            }
        )
        .set_properties(
            tables[7].columns,
            **{
                "background-color": "#E6B8B7",
                "border": "1px solid #963634",
            }
        )
        .set_properties(
            tables[8].columns,
            **{
                "background-color": "#F2DCDB",
                "border": "1px solid #DA9694",
            }
        )
        .set_properties(
            tables[9].columns,
            **{
                "background-color": "#EBF1DE",
                "border": "1px solid #C4D79B",
            }
        )
        .set_properties(
            tables[10].columns,
            **{
                "background-color": "#D8E4BC",
                "border": "1px solid #A4C163",
            }
        )
        .set_properties(
            tables[11].columns,
            **{
                "background-color": "#C4D79B",
                "border": "1px solid #76933C",
            }
        )
        .set_properties(
            tables[12].columns,
            **{
                "background-color": "#A4C163",
                "border": "1px solid #4F6228",
            }
        )
        .set_properties(
            tables[13].columns,
            **{
                "background-color": "#F7C7AC",
                "border": "1px solid #BE5014",
            }
        )
        .set_properties(
            tables[14].columns,
            **{
                "background-color": "#E49EDD",
                "border": "1px solid #782170",
            }
        )
        .set_properties(
            tables[15].columns,
            **{
                "background-color": "#F2CEEF",
                "border": "1px solid #D86DCD",
            }
        )
    )

    return styled_df


def create_styled_metadata(tables: list[pd.DataFrame], metadata: pd.DataFrame) -> Styler:
    styled_df = (
        metadata.style
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(["Year", "ID"]), :], # type: ignore
            **{
                "background-color": "#D9D9D9",
                "border": "2px solid #A6A6A6",
                "font-weight": "bold",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[0].columns), slice(None)], # type: ignore
            **{
                "background-color": "#DAEEF3",
                "border": "1px solid #92CDDC",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[1].columns), slice(None)], # type: ignore
            **{
                "background-color": "#DDD9C4",
                "border": "1px solid #948A54",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[2].columns), slice(None)], # type: ignore
            **{
                "background-color": "#DCE6F1",
                "border": "1px solid #95B3D7",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[3].columns), slice(None)], # type: ignore
            **{
                "background-color": "#B8CCE4",
                "border": "1px solid #95B3D7",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[4].columns), slice(None)], # type: ignore
            **{
                "background-color": "#DCE6F1",
                "border": "1px solid #95B3D7",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[5].columns), slice(None)], # type: ignore
            **{
                "background-color": "#B8CCE4",
                "border": "1px solid #95B3D7",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[6].columns), slice(None)], # type: ignore
            **{
                "background-color": "#DA9694",
                "border": "1px solid #632523",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[7].columns), slice(None)], # type: ignore
            **{
                "background-color": "#E6B8B7",
                "border": "1px solid #963634",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[8].columns), slice(None)], # type: ignore
            **{
                "background-color": "#F2DCDB",
                "border": "1px solid #DA9694",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[9].columns), slice(None)], # type: ignore
            **{
                "background-color": "#EBF1DE",
                "border": "1px solid #C4D79B",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[10].columns), slice(None)], # type: ignore
            **{
                "background-color": "#D8E4BC",
                "border": "1px solid #A4C163",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[11].columns), slice(None)], # type: ignore
            **{
                "background-color": "#C4D79B",
                "border": "1px solid #76933C",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[12].columns), slice(None)], # type: ignore
            **{
                "background-color": "#A4C163",
                "border": "1px solid #4F6228",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[13].columns), slice(None)], # type: ignore
            **{
                "background-color": "#F7C7AC",
                "border": "1px solid #BE5014",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[14].columns), slice(None)], # type: ignore
            **{
                "background-color": "#E49EDD",
                "border": "1px solid #782170",
            }
        )
        .set_properties(
            pd.IndexSlice[metadata["Name"].isin(tables[15].columns), slice(None)], # type: ignore
            **{
                "background-color": "#F2CEEF",
                "border": "1px solid #D86DCD",
            }
        )
    )

    return styled_df


def write_excel(year: int) -> None:
    with open(Path(__file__).parent.joinpath("v0_metadata.yaml"), encoding="utf8") as file:
        metadata = yaml.safe_load(file)
    metadata_table = pd.DataFrame(metadata["columns"])
    columns = [item["Name"] for item in metadata["columns"]]
    tables = create_tables(year)
    for table in tables:
        table.columns = table.columns.map(lambda n: n.replace("_", " "))
    styled_data = create_styled_table(tables, columns=columns)
    styled_metadata = create_styled_metadata(tables, metadata_table)
    with pd.ExcelWriter(f"Iran_HBS_Summary_{year}_V.0.{SUB_VERSION}.xlsx") as excel_writer:
        styled_data.to_excel(
            excel_writer,
            sheet_name="Data",
            merge_cells=False,
            index=False,
            freeze_panes=(1, 2)
        )
        styled_metadata.to_excel(
            excel_writer,
            sheet_name="Metadata",
            index=False,
            freeze_panes=(1, 0),
        )


def write_csv(year: int) -> None:
    with open(Path(__file__).parent.joinpath("v0_metadata.yaml"), encoding="utf8") as file:
        metadata = yaml.safe_load(file)
    metadata_table = pd.DataFrame(metadata["columns"])
    tables = create_tables(year)
    summary_table = create_summary_table(tables)
    metadata_buffer = io.BytesIO()
    styled_metadata = create_styled_metadata(tables, metadata_table)
    styled_metadata.to_excel(
        metadata_buffer,
        sheet_name="Metadata",
        index=False,
        freeze_panes=(1, 0),
    )
    data_buffer = io.BytesIO()
    summary_table.to_csv(data_buffer, index=False, encoding="iso-8859-1")
    with ZipFile(
        f"Iran_HBS_Summary_{year}_V.0.{SUB_VERSION}.zip",
        mode="w",
        compression=ZIP_DEFLATED,
    ) as zip_file:
        zip_file.writestr("Metadata.xlsx", metadata_buffer.getvalue())
        zip_file.writestr("Data.csv", data_buffer.getvalue())



"""
Schooling Years Calculation Module

This module calculates schooling years based on education levels and age,
considering both the theoretical value of degrees and physical constraints.
It handles special cases like students, primary education, and accelerated learners.

Key Concepts:
- Degree_Worth_Years: Theoretical years the degree represents
- Max_Possible_Years: Maximum physically possible years given age (starting at 6)
- Years_of_Schooling: Main calculation considering student status
- Conservative_Years_of_Schooling: Age-constrained version for reality checks
"""

import pandas as pd
import numpy as np


# Education level to typical years mapping
# Represents the theoretical years of schooling each degree represents
EDUCATION_MAPPING = {
    "Primary": 6,                   # 6 years of primary education
    "Lower_Secondary": 9,           # 3 years after primary (6+3=9)
    "Higher_Secondary": 12,         # 3 years after lower secondary (9+3=12)
    "Pre_University": 12,           # Same as high school completion
    "Short_Cycle_Tertiary": 14,     # 2 years after diploma (12+2=14)
    "Bachelors": 16,                # 4 years after diploma (12+4=16)
    "Masters": 19,                  # 2 years after bachelor's (16+3=19)
    "Doctoral": 24,                 # 4 years after master's (19+5=24)
    "Unofficial": 3,                # Non-formal education
}

# Starting age for formal education
MIN_AGE = 6


def calculate_years_of_schooling(table: pd.DataFrame) -> pd.DataFrame:
    """
    Calculate schooling years considering education levels, age, and student status.

    Parameters:
    -----------
    table : pd.DataFrame
        Input dataframe containing:
        - Education_Level: Categorical education levels
        - Age: Integer age of individuals
        - Is_Student: Boolean indicating current student status
        - Year, ID, Member_Number: Identifying columns

    Returns:
    --------
    pd.DataFrame
        Dataframe with schooling years calculations and analysis flags
    """

    # Input validation
    required_columns = [
        "Year",
        "ID",
        "Member_Number",
        "Education_Level",
        "Age",
        "Is_Student",
    ]
    missing_columns = [col for col in required_columns if col not in table.columns]
    if missing_columns:
        raise ValueError(f"Missing required columns: {missing_columns}")

    return (
        table
        # Calculate base metrics
        .assign(
            # Theoretical years the education level represents
            Degree_Worth_Years=lambda df: df["Education_Level"].map(EDUCATION_MAPPING),

            # Maximum physically possible years given age (starting at MIN_AGE)
            Max_Possible_Years=lambda df: (df["Age"] - MIN_AGE).clip(0),

            # Calculate schooling years with special cases
            Years_of_Schooling=lambda df: _calculate_main_years_of_schooling(df),

            # Analysis flags for data quality and acceleration
            Years_Accelerated=lambda df: 
            df["Years_of_Schooling"].sub(df["Max_Possible_Years"]).clip(0),
            Is_Accelerated=lambda df: df["Years_Accelerated"].gt(0),

            # Conservative estimate respecting age constraints
            Conservative_Years_of_Schooling=lambda df:
            df[["Years_of_Schooling", "Max_Possible_Years"]].min(axis=1),
        )
    )


def _calculate_main_years_of_schooling(df):
    """
    Internal function to calculate main schooling years based on education rules.

    Rules:
    - Students get one less year than their current level (haven't completed)
    - Primary non-students with limited age get age-based calculation (dropouts)
    - Other non-students get full degree worth years
    """
    return np.where(
        # Students: subtract 1 year since they haven't completed current level
        df["Is_Student"].eq(True),
        (df["Degree_Worth_Years"] - 1).clip(0, df["Max_Possible_Years"]),

        # Non-students
        np.where(
            # Primary non-students who are young: use age-based (likely dropouts)
            df["Education_Level"].eq("Primary")
            & df["Max_Possible_Years"].lt(EDUCATION_MAPPING["Primary"]),
            df["Max_Possible_Years"],

            # Other non-students: completed education, use degree worth
            df["Degree_Worth_Years"],
        ),
    )



import pandas as pd
from pandas.api.types import CategoricalDtype

from ..api import api


def number_of_members(table: pd.DataFrame) -> pd.DataFrame:
    return (
        table.assign(Adult=lambda df: df["Age"] >= 14)
        .groupby(["Year", "ID"], as_index=False)
        .agg(
            Members=pd.NamedAgg("Member_Number", "count"),
            Adults=pd.NamedAgg("Adult", "sum"),
        )
        .assign(Childs=lambda df: df["Members"] - df["Adults"])
    )


def equivalence_scale(table: pd.DataFrame) -> pd.DataFrame:
    return table.assign(
        Household=1,
        Per_Capita=table["Members"],
        Family_Size=table["Members"],
        OECD=table["Adults"].multiply(0.7).add(0.3).add(table["Childs"].multiply(0.5)),
        OECD_Modified=table["Adults"].multiply(0.5).add(0.5)
        .add(table["Childs"].multiply(0.3)),
        Square_Root=table["Members"].pow(1/2),
    )


def adjust_month(table: pd.DataFrame) -> pd.DataFrame:
    table["Month"] = table["Month"].replace({1: 13}).sub(1)
    return table


def create_season_number(table: pd.DataFrame) -> pd.DataFrame:
    table["Season_Number"] = table["Month"].sub(1).floordiv(3).add(1)
    return table


def season_name(table: pd.DataFrame) -> pd.DataFrame:
    table["Season"] = (
        table["Season_Number"]
        .astype(CategoricalDtype([1, 2, 3, 4], ordered=True))
        .cat.rename_categories({1: "Spring", 2: "Summer", 3: "Autumn", 4: "Winter"})
    )
    return table


def calculate_amount_before_1382(table: pd.DataFrame) -> pd.DataFrame:
    if "Price" not in table.columns:
        table["Amount"] = table["Kilos"]
        return table
    table["Amount"] = (
        table["Kilos"]
        .mask(
            table["Kilos"].isna() & table["Price"].notna(),
            table["Expenditure"] / table["Price"],
        )
    )
    return table


def calculate_amount_after_1383(table: pd.DataFrame) -> pd.DataFrame:
    table["Amount"] = table["Kilos"].fillna(0) + table["Grams"].fillna(0).div(1_000)
    filt = table[["Grams", "Kilos"]].notna().max(axis="columns").eq(False)
    table.loc[filt, "Amount"] = None
    filt = table["Amount"].isna() & table["Price"].gt(0)
    table["Amount"] = table["Amount"].mask(filt, table["Expenditure"]/ table["Price"])
    return table


def add_duration_before_1368(table: pd.DataFrame) -> pd.DataFrame:
    return (
        table
        .pipe(api.add_attribute, name="Urban_Rural")
        .pipe(api.add_classification, name="Duration")
        .assign(
            Duration=lambda df:
            df["Rural_Duration"]
            .where(df["Urban_Rural"].eq("Rural"), df["Urban_Duration"])
        )
    )



"""HBSIR, Household Budget Survey of Iran, Main API

This module provides a high-level API for accessing and analyzing data from the
Household Budget Survey of Iran (HBSIR). It simplifies data loading, exploration,
and analysis for users without extensive programming knowledge.

"""
# pylint: disable=too-many-arguments
# pylint: disable=unused-argument
# pylint: disable=too-many-locals

from typing import Any, Iterable, Literal, overload, Optional
from pathlib import Path

import pandas as pd

from bssir.metadata_reader import config, _Years
from bssir.api import API, _DataSource, _Frequency, _SeparateBy


defaults, metadata = config.set_package_config(Path(__file__).parent)
api = API(defaults=defaults, metadata=metadata)

_Table = Literal[
    "household_information",
    "members_properties",
    "house_specifications",
    "old_rural_house_specifications",
    "old_urban_house_specifications",
    "food",
    "tobacco",
    "cloth",
    "home",
    "furniture",
    "medical",
    "transportation",
    "communication",
    "entertainment",
    "education",
    "hotel",
    "miscellaneous",
    "durable",
    "investment",
    "employment_income",
    "self_employed_income",
    "other_income",
    "subsidy",
    "old_rural_public_employment_income",
    "old_urban_public_employment_income",
    "old_rural_private_employment_income",
    "old_urban_private_employment_income",
    "old_agricultural_self_employed_income",
    "old_non_agricultural_self_employed_income",
    "old_other_income",
    "Season",
    "Weight",
    "Number_of_Members",
    "Adult_Equivalent_Scale",
    "Equivalence_Scale",
    "Years_of_Schooling",
    "Original_Expenditures",
    "Expenditures",
    "Total_Expenditure",
    "Original_Outlays",
    "Outlays",
    "Total_Outlay",
    "Imputed_Rent",
    "Income_Breakdown",
    "Members_Income_Breakdown",
    "Total_Income",
    "Members_Total_Income",
]
_ExternalTable = Literal[
    "CPI",
]
_Attribute = Literal["Urban_Rural", "Province", "County"]


def __get_optional_params(local_variables: dict) -> dict:
    return {key: value for key, value in local_variables.items() if value is not None}


def load_table(
    table_name: _Table | str,
    years: _Years = "last",
    form: Literal["normalized", "cleaned", "raw"] | None = None,
    *,
    on_missing: Literal["error", "download", "create"] | None = None,
    redownload: bool | None = None,
    save_downloaded: bool | None = None,
    recreate: bool | None = None,
    save_created: bool | None = None,
) -> pd.DataFrame:
    """Load a table for the given table name and year(s).

    This function loads original survey tables as well as package
    tables defined in this library.

    Original survey tables are available in three forms:

    - 'raw' - Contains the raw survey data without any modifications
    - 'cleaned' - Raw data with added labels, types, removed irrelevant
        values, but no changes to actual data
    - 'normalized' - Standardized data form with consistent column
        names, value encodings and table structure applied across data
        from multiple survey years. Also adds useful metadata like
        table name and year identifiers.

    Package tables are defined in this library to facilitate working
    with the data and are only available in normalized form.

    For more details on available tables, see the documentation
    [tables page](https://iran-open-data.github.io/HBSIR/tables/).

    Parameters
    ----------
    table_name : _Table, default "data"
        The name of the table to load.
    years : _Years, default "last"
        The years of data to load.
    form : {"normalized", "cleaned", "raw"}, default "normalized"
        The form of the data to load. Options are "normalized",
        "cleaned", or "raw".

    !!! note
        The `years` parameter accepts different input types:

        - int: A single year integer like 1390 or 90
        - list[int]: A list of integer years [1390, 1395, 1400]
        - str: A hyphenated string range like '1390-1395'
        - "all": A string indicating all available years
        - "last": A string indicating only the most recent year

    Other parameters
    ----------------
    on_missing : {"download", "create", "error"}, default "download"
        Behavior if table is missing. "download" downloads the table,
        "create" generates table from raw data, "error" raises an
        exception.
    redownload : bool, default False
        Whether to re-download table if it exists.
    save_downloaded : bool, default True
        Whether to save newly downloaded data.
    recreate : bool, default False
        Whether to recreate table if it exists.
    save_created : bool, default True
        Whether to save newly created data.

    Returns
    -------
    pd.DataFrame
        Loaded table as a pandas DataFrame.

    Raises
    ------
    FileNotFoundError
        If data is missing and on_missing='error'.


    Examples
    --------
    ``` python
    import hbsir
    table = hbsir.load_table("Expenditures", 1400) # Loads expenditures for year 1400
    ```

    """
    parameters = __get_optional_params(locals())
    return api.load_table(**parameters)


def load_external_data(
    table_name: _ExternalTable | str,
    data_source: Optional[_DataSource] = None,
    frequency: Optional[_Frequency] = None,
    separate_by: Optional[_SeparateBy] = None,
    form: Optional[Literal["cleaned", "original"]]  = None,
    on_missing: Optional[Literal["error", "download", "create"]]  = None,
    save_downloaded: Optional[bool]  = None,
    redownload: Optional[bool]  = None,
    save_created: Optional[bool]  = None,
    recreate: Optional[bool]  = None,
    reset_index: bool = True,
) -> pd.DataFrame:
    parameters = __get_optional_params(locals())
    return api.load_external_table(**parameters)


def load_knowledge(
    name: str,
    years: _Years | None = None,
    urban_rural: Literal["urban", "rural"] | None = None,
) -> Any:
    """Retrieve tables and graphs stored in the library's Knowledge Base.

    This function allows users to access valuable information, including tables and
    graphs, created from survey data and drawn from associated articles and reports.
    The data is pre-processed, organized, and readily available in the library's
    Knowledge Base, reducing the necessity for extensive data processing.


    Parameters
    ----------
    name : str
        The name of artifact requested.
    years : _Years, optional
        The years for which knowledge data is requested.

    Returns
    -------
    Any
        The requested knowledge data, which includes tables, graphs, or other informative content.

    """
    kwargs = {}
    if urban_rural is not None:
        kwargs["urban_rural"] = urban_rural
    return api.load_knowledge(name=name, years=years, **kwargs)


def add_attribute(
    table: pd.DataFrame,
    name: _Attribute | str,
    *,
    aspects: Iterable[str] | str | None = None,
    column_names: Iterable[str] | str | None = None,
    id_col: str | None = None,
    year_col: str | None = None,
) -> pd.DataFrame:
    """Add attributes to table based on ID column.

    This function takes a Pandas DataFrame containing a household ID column,
    and enriches it by adding columns indicating attributes like urban/rural
    status, province, county etc.

    Parameters
    ----------
    table : pd.DataFrame
        Input DataFrame containing the ID column.
    name : _Attribute
        Name of attribute to add.
    aspects : list of str, default "name"
        Specific aspects of the attribute to add as columns.

    !!! note
        Supported attribute names are:

        - "Urban_Rural": Urban or rural status (aspects: "name", "farsi_name")
        - "Province": Province name (aspects: "name", "farsi_name")
        - "County": County name (aspects: "name", "farsi_name")

    Other parameters
    ----------------
    column_names : list of str, optional
        Custom names to use for the added columns.
    id_col : str, default "ID"
        Name of the ID column in the table.
    year_col : str, default "Year"
        Name of the year column in the table.

    Returns
    -------
    pd.DataFrame
        Input DataFrame with additional columns containing attribute values.

    Examples
    --------
    ``` python
    import hbsir
    table = hbsir.load_table(years=1401)
    table = add_attribute(table, "Urban_Rural")
    table = add_attribute(table, "Province", aspects="farsi_name")
    ```

    """
    parameters = __get_optional_params(locals())
    return api.add_attribute(**parameters)


def add_classification(
    table: pd.DataFrame,
    target: str | None = None,
    *,
    name: str | None = None,
    aspects: Iterable[str] | str | None = None,
    levels: Iterable[int] | int | None = None,
    column_names: Iterable[str] | str | None = None,
    year_col: str | None = None,
) -> pd.DataFrame:
    """Add commodity, industry or occupation classification to table.

    This function takes a Pandas DataFrame containing COICOP commodity codes,
    ISIC industry codes or ISCO occupation codes, and enriches it by decoding
    these codes into descriptive categories.

    The output DataFrame will contain additional columns with attributes
    from the classification system at the specified hierarchical levels,
    providing more information alongside the original coded values.

    Parameters
    ----------
    table : pd.DataFrame
        Input DataFrame containing the classification code column.
    target : str
        Name of the column containing industry or occupation codes.
    name : str, default 'original'
        Name of classification to apply.

    Other parameters
    ----------------
    aspects : list of str, optional
        Classification aspects to add as columns, like "label", "farsi_label".
    levels : list of int, optional
        Hierarchy levels to include, like 1 or [1, 2].
    column_names : list of str, optional
        Custom names to use for the added columns.
    year_col : str, default "Year"
        Name of year column for resolving annual changes.

    Returns
    -------
    pd.DataFrame
        Input DataFrame with additional columns containing classification
            attributes.

    Examples
    --------
    ```python
    import hbsir
    table = hbsir.load_table("Expenditures", 1401)
    table = hbsir.add_classification(table)
    ```

    """
    parameters = __get_optional_params(locals())
    return api.add_classification(**parameters)


def add_weight(table: pd.DataFrame) -> pd.DataFrame:
    """Add sampling weights to the table.

    Loads appropriate sample weights for each year in the table and merges
    them onto the table.

    Weights for years prior to 1395 are loaded from external parquet data,
    while weights for 1395 onward come from the household_information table.

    Parameters
    ----------
    table : pd.DataFrame
        Input data table, containing a column of year values

    Returns
    -------
    table : pd.DataFrame
        Input `table` with 'Weight' column added

    """
    return api.add_weight(table)


@overload
def setup(
    years: _Years,
    *,
    table_names: Optional[str | list[str]] = None,
    replace: bool = False,
    method: Literal["create_from_raw"] = "create_from_raw",
    download_source: Literal["original", "mirror"] | str = "original",
) -> None:
    ...


@overload
def setup(
    years: _Years,
    *,
    table_names: Optional[str | list[str]] = None,
    replace: bool = False,
    method: Literal["download_cleaned"] = "download_cleaned",
    download_source: Literal["mirror"] | str = "mirror",
) -> None:
    ...


def setup(
    years: _Years,
    *,
    table_names: Optional[str | list[str]] = None,
    replace: Optional[bool] = None,
    method: Optional[Literal["create_from_raw", "download_cleaned"]] = None,
    download_source: Optional[Literal["original", "mirror"] | str] = None,
) -> None:
    """Set up package data for the given years.

    This function handles downloading or generating the required data tables
    to enable full functionality of the package for the specified years.

    By default, it tries to download pre-cleaned data which allows the package
    to work out of the box. But advanced users can generate data locally
    from raw survey files for more control, transparency or customization.

    Parameters
    ----------
    years : _Years
        The years of data to set up.

    Other parameters
    ----------------
    replace : bool, default False
        Whether to re-download or re-generate data if it already exists.
    method : {"create_from_raw", "download_cleaned"}, default "create_from_raw"
        The method to use for setting up data.
    download_source : {"original", "google_drive", "mirror"}, default "original"
        Where to download data from.

    Returns
    -------
    None
        The function executes setup tasks but does not return anything.

    Examples
    --------
    Basic usage:

    ```python
    import lfsir

    lfsir.setup(years="1390-1400")
    ```

    Advanced usage:

    ```python
    lfsir.setup(years="1390-1400",
               method="create_from_raw",
               replace=True)
    ```

    This will recreate all tables from 1390 to 1400 from raw data even if they
    already exist.

    """
    parameters = __get_optional_params(locals())
    api.setup(**parameters)
    years_list = set(api.utils.parse_years(years))
    if years_list.intersection(set(range(1387, 1392))):
        api.load_external_table("hbsir_counties")
    if years_list.intersection(set(range(1376, 1396))):
        api.load_external_table("hbsir_weights")


def setup_config(
    mode: Literal['Standard', 'Colab'] = "Standard",
    replace: bool = False
) -> None:
    """Copy default config file to data directory.

    Copies the default config file 'settings_sample.yaml' from the package
    directory to 'config/lfsir_settings.yaml' in the root data directory.

    Overwrites any existing file if replace=True.

    Parameters
    ----------
    replace : bool, default False
        Whether to overwrite existing config file.

    """
    global defaults, metadata, api
    api.setup_config(mode=mode, replace=replace)
    defaults, metadata = config.set_package_config(Path(__file__).parent)
    api = API(defaults=defaults, metadata=metadata)



"""Common functions for all tables"""

import pandas as pd
import numpy as np

from ...api import api

CUT_BINS = {
    "urban": {
        1389: [-np.inf, 12, 16.5, 19.5, 24, 30, 45, 60, 75, 90, np.inf],
        1401: [-np.inf, 120, 195, 270, 360, 480, 600, 720, 900, 1200, np.inf],
    },
    "rural": {
        1389: [-np.inf, 7.2, 9, 12, 16.5, 19.5, 24, 30, 45, 60, np.inf],
        1401: [-np.inf, 75, 120, 195, 270, 360, 480, 600, 720, 900, np.inf],
    },
}


def create_expenditure_tiers(*, urban_rural: str, year: int):
    """Create expenditure tiers"""
    urban_rural = urban_rural.lower()
    bins = CUT_BINS[urban_rural][year]
    return pd.cut(
        api.load_table("Total_Expenditure", years=year)
        .set_index(["Year", "ID"])
        .loc[:, "Net_Expenditure"]
        .div(1e6),
        bins=bins,
        labels=(
            [f"<{bins[1]}"]
            + [f"{lower}-{higher}" for lower, higher in zip(bins[1:-2], bins[2:-1])]
            + [f"{bins[-2]}<"]
        ),
    ).rename("Expenditure_Tier")


def filter_urban_rural(table: pd.DataFrame, urban_rural: str) -> pd.DataFrame:
    """Filter to urban or rural"""
    urban_rural = urban_rural.lower()
    if urban_rural == "urban":
        table = (
            table.pipe(api.add_attribute, name="Urban_Rural")
            .loc[lambda df: df["Urban_Rural"] == "Urban"]
            .drop(columns="Urban_Rural")
        )
    elif urban_rural == "rural":
        table = (
            table.pipe(api.add_attribute, name="Urban_Rural")
            .loc[lambda df: df["Urban_Rural"] == "Rural"]
            .drop(columns="Urban_Rural")
        )
    return table



"""HBSIR - Tools for working with Household Budget Survey of Iran

This package provides a set of tools to facilitate working with the 
Household Budget Survey of Iran (HBSIR) data.
  
It allows easy access to standardized survey tables across multiple 
years. It also enables data enrichment, aggregation, analysis and 
visualization.

"""

from .api import (
    load_table,
    load_external_data,
    load_knowledge,
    add_attribute,
    add_classification,
    add_weight,
    setup,
    setup_config,
)

from . import calculate


__version__ = "0.6.6"

__all__ = [
    "load_table",
    "load_external_data",
    "load_knowledge",
    "add_attribute",
    "add_classification",
    "add_weight",
    "setup",
    "setup_config",
    "calculate",
]



from typing import Literal

import pandas as pd

from ...api import api

from .common import create_expenditure_tiers, filter_urban_rural


def create_table_for_a_year(
    year: int, urban_rural: Literal["urban", "rural"]
) -> pd.DataFrame:
    head_state = (
        api.load_table("members_properties", years=1401)
        .set_index(["Year", "ID"])
        .loc[lambda df: df["Relationship"] == "Head", ["Activity_Status"]]
    )

    table = pd.concat(
        [
            head_state,
            create_expenditure_tiers(urban_rural=urban_rural, year=year),
            api.load_table("Weight", years=year).set_index(["Year", "ID"])["Weight"],
        ],
        axis="columns",
    )
    table = filter_urban_rural(table, urban_rural)

    weight_table = (
        table.groupby(["Activity_Status", "Expenditure_Tier"], observed=True)["Weight"]
        .sum()
        .unstack()
    )
    index_weight_sum = weight_table.sum(axis="index")
    columns_weight_sum = weight_table.sum(axis="columns")
    col_sum = columns_weight_sum.div(columns_weight_sum.sum()).mul(100).rename("Total")
    main_table = weight_table.div(index_weight_sum).mul(100)
    main_table = pd.concat([col_sum, main_table], axis="columns")
    main_table = pd.concat(
        [
            main_table.sum().to_frame("Total").T,
            main_table,
        ]
    )

    main_table = main_table.fillna(0)
    main_table.index = main_table.index.astype(str)
    return main_table


def main(years: list[int], urban_rural: Literal["urban", "rural"]):
    if len(years) == 1:
        return create_table_for_a_year(years[0], urban_rural)
    raise ValueError
